@@ -17,17 +17,6 @@
 #	define PGRN_SUPPORT_WAL
 #endif
 
-#if PG_VERSION_NUM < 110000
-#	define PG_GETARG_JSONB_P(n) PG_GETARG_JSONB((n))
-#	define DatumGetJsonbP(datum) DatumGetJsonb((datum))
-#endif
-
-#if PG_VERSION_NUM >= 110000
-typedef const char *PGrnStringOptionValue;
-#else
-typedef char *PGrnStringOptionValue;
-#endif
-
 #if PG_VERSION_NUM >= 120000
 #	define PGRN_SUPPORT_INDEX_CLAUSE
 #	define PGRN_SUPPORT_TABLEAM
@@ -70,11 +59,11 @@ typedef char *PGrnStringOptionValue;
 						   (index),					\
 						   (indexInfo),				\
 						   (allowSync),				\
-						   false,					\
+						   true,					\
 						   (callback),				\
 						   (callbackState),			\
 						   NULL)
-#elif PG_VERSION_NUM >= 110000
+#else
 #	define PGrnIndexBuildHeapScan(heap,				\
 								  index,			\
 								  indexInfo,		\
@@ -88,19 +77,6 @@ typedef char *PGrnStringOptionValue;
 					   (callback),					\
 					   (callbackState),				\
 					   NULL)
-#else
-#	define PGrnIndexBuildHeapScan(heap,				\
-								  index,			\
-								  indexInfo,		\
-								  allowSync,		\
-								  callback,			\
-								  callbackState)	\
-	IndexBuildHeapScan((heap),						\
-					   (index),						\
-					   (indexInfo),					\
-					   (allowSync),					\
-					   (callback),					\
-					   (callbackState))
 #endif
 
 #if PG_VERSION_NUM >= 120000
@@ -187,32 +163,16 @@ typedef char *PGrnStringOptionValue;
 	CreateTemplateTupleDesc((natts), false)
 #endif
 
-#if PG_VERSION_NUM < 110000
-#	define PG_RETURN_JSONB_P(x) PG_RETURN_JSONB(x)
-#endif
-
 #if PG_VERSION_NUM >= 120000
 #	define PGRN_HAVE_TUPLE_TABLE_SLOT_TABLE_OID
 #endif
 
-#if PG_VERSION_NUM < 110000
-#	define PGrnBackgroundWorkerInitializeConnection(dbname, username, flags) \
-	BackgroundWorkerInitializeConnection((dbname), (username))
-#	define PGrnBackgroundWorkerInitializeConnectionByOid(dboid, useroid, flags) \
-	BackgroundWorkerInitializeConnectionByOid((dboid), (useroid))
-#else
-#	define PGrnBackgroundWorkerInitializeConnection(dbname, username, flags) \
-	BackgroundWorkerInitializeConnection((dbname), (username), (flags))
-#	define PGrnBackgroundWorkerInitializeConnectionByOid(dboid, useroid, flags) \
-	BackgroundWorkerInitializeConnectionByOid((dboid), (useroid), (flags))
-#endif
-
 #if PG_VERSION_NUM >= 120000
 #	define PGRN_FORM_PG_DATABASE_HAVE_OID
 #endif
 
-#if PG_VERSION_NUM >= 110000
-#	define PGRN_BACKGROUND_WORKER_HAVE_BGW_TYPE
+#if PG_VERSION_NUM >= 120000
+#	define PGRN_SUPPORT_PROGRESS
 #endif
 
 #if PG_VERSION_NUM >= 130000
@@ -223,4 +183,44 @@ typedef char *PGrnStringOptionValue;
 
 #if PG_VERSION_NUM >= 130000
 #	define PGRN_HAVE_COMMON_HASHFN_H
+#	define PGRN_INDEX_AM_ROUTINE_HAVE_AM_USE_MAINTENANCE_WORK_MEM
+#	define PGRN_INDEX_AM_ROUTINE_HAVE_AM_PARALLEL_VACUUM_OPTIONS
+#endif
+
+#if PG_VERSION_NUM >= 160000
+#	define PGRN_RELATION_GET_LOCATOR(relation) ((relation)->rd_locator)
+#	define PGRN_RELATION_GET_LOCATOR_NUMBER(relation)	\
+	((relation)->rd_locator.relNumber)
+#	define PGRN_RELATION_GET_LOCATOR_SPACE(relation)	\
+	((relation)->rd_locator.spcOid)
+#	include <common/relpath.h>
+typedef RelFileNumber PGrnRelFileNumber;
+#	define PGrnRelidByRelfilenumber(tablespaceOid, fileNumber)	\
+	RelidByRelfilenumber((tablespaceOid), (fileNumber))
+#	define pgrn_pg_tablespace_ownercheck(tablespaceOid, userOid)	\
+	object_ownercheck(TableSpaceRelationId, (tablespaceOid), (userOid))
+#	define pgrn_pg_class_ownercheck(relationOid, userOid)	\
+	object_ownercheck(RelationRelationId, (relationOid), (userOid))
+#else
+#	define PGRN_RELATION_GET_LOCATOR(relation) ((relation)->rd_node)
+#	define PGRN_RELATION_GET_LOCATOR_NUMBER(relation)	\
+	((relation)->rd_node.relNode)
+#	define PGRN_RELATION_GET_LOCATOR_SPACE(relation)	\
+	((relation)->rd_node.spcNode)
+typedef Oid PGrnRelFileNumber;
+#	define PGrnRelidByRelfilenumber(tablespaceOid, fileNumber)	\
+	RelidByRelfilenode((tablespaceOid), (fileNumber))
+#	define pgrn_pg_tablespace_ownercheck(tablespaceOid, userOid)	\
+	pg_tablespace_ownercheck((tablespaceOid), (userOid))
+#	define pgrn_pg_class_ownercheck(relationOid, userOid)	\
+	pg_class_ownercheck((relationOid), (userOid))
+#endif
+
+#if PG_VERSION_NUM >= 150000
+#	define PGRN_RELKIND_HAS_PARTITIONS(relkind)	\
+	RELKIND_HAS_PARTITIONS(relkind)
+#else
+#	define PGRN_RELKIND_HAS_PARTITIONS(relkind)	\
+	((relkind) == RELKIND_PARTITIONED_TABLE ||	\
+	 (relkind) == RELKIND_PARTITIONED_INDEX)
 #endif
